function [wmCoeff,vNodeX,vNodeY,nodeX,nodeY] =...
    CervicalVertebraBloodVesselMove(dt, angle)
%[wmCoeff,vNodeX,vNodeY,nodeX,nodeY] =...
%                                CervicalVertebraBloodVesselMove(dt)
%[wmCoeff,vNodeX,vNodeY,nodeX,nodeY] =...
%                                CervicalVertebraBloodVesselMove(dt, angle)
% Input;
%  dt: time step size [ms]
%  angle: a time_steps*7 matrix, which stores the rotational angle of
%  cervical vertebrae c1, c2, c3, c4, c5, c6 and T1.
% Output:
%  vmCoeff: coefficients of 5 order polynomial equations which describe the
%  shape of the blood vessel at each time step
%  vNodeX: x coordinate of the nodes on the blood vessel
%  vNodeY: y coordinate of the nodes on the blood vessel
%  nodeX: x coordinate of the nodes on the cervical vertebrae
%  nodeY: y coordinate of the nodes on the cervical vertebrae
%
% This function describes the shape of the boold vessel attached on the
% cervical vertebrae at each time step as a 5 order ploynomial equation. 
% Both the coordinates of the key nodes and the equation coefficients are 
% returned and saved in the files. The definition of key nodes refers to 
% Liu,F. and Yang,J. "The modeling of Whiplash Injuries in CFD". 
%
% This function is developed in September 2007
%  Feng Liu
%  Vehicle Safety Division
%  Chalmers University of Technology
%  liu.peak@gmail.com
% Revisions
%--------------------
% February 2009, add comments, modified from a .m file to a function

if nargin == 1
    load angle.mat -ascii
end

% Initialize coordinate, unit [m]
tV1 = 10.8e-3; % higth of T1 / 2 + disc7
d = 20e-3;     % distance between cervical vertebrae and blood vessel

% nodeo=[d 0];

% node matrix stores the position of each key node at initial position
node(1,1,:) = [ d  8.6e-3];          % [x y]
node(1,2,:) = [ d  23.8e-3+tV1];
node(1,3,:) = [ d  40.78e-3+tV1];
node(1,4,:) = [ d  59.22e-3+tV1];
node(1,5,:) = [ d  77.67e-3+tV1];
node(1,6,:) = [ d  99.22e-3+tV1];
node(1,7,:) = [ d  108e-3+tV1];

dis = zeros(1,7);  % distance between each key node
dis(1) = node(1,1,2);
for iNode = 2 : 7
    nod0 = [node(1,iNode-1,1) node(1,iNode-1,2)];
    nod1 = [node(1,iNode,1) node(1,iNode,2)];
    dis(iNode) = Distance(nod1,nod0);
end
dis(7) = dis(6) + dis(7);

sT = 0;       % startTime = 0ms
eT = 300;     % endTime   = 300ms

% Iinitialize nodes position on the blood vessel "vNode"
vNode=zeros((eT-sT)/dt,7,2);
vNode(1,1,:)=[0 0];
for id=2:7
    vNode(1,id,1)=node(1,id,1)-d;
    vNode(1,id,2)=node(1,id,2);
end

% Equation coefficient for the first time step
ft_ = fittype('poly5' );
wmCoeff = zeros((eT-sT)/dt,6);    % whiplash motion coefficient
ver = zeros(1,7);
hor = zeros(1,7);

coeffs = zeros(1,6);
for index = 1 : length(coeffs)
    wmCoeff(1,index) = coeffs(index);
end

% Calculate coordinates of the each node at each time step
for iTimeStep = 2 : (eT-sT)/dt
    
%     cur_time=sT+dt*(iTimeStep-1);
    
    node(iTimeStep,1,1) = node(1,1,1);
    node(iTimeStep,1,2) = node(1,1,2);
    
    theta = angle(iTimeStep,:)*pi/180;
    
    for iNode = 2 : 6
        cen_x = node(iTimeStep,iNode-1,1);     % rotation center
        cen_y = node(iTimeStep,iNode-1,2);
        [new_bx,new_by] = VertebraMove(theta(iNode),cen_x,cen_y,dis(iNode));
        node(iTimeStep,iNode,1) = new_bx;
        node(iTimeStep,iNode,2) = new_by;
        
        [new_vx, new_vy] = VesselMove(new_bx,new_by,theta(iNode),d);
        vNode(iTimeStep,iNode,1) = new_vx;
        vNode(iTimeStep,iNode,2) = new_vy;
    end
   
    % Calculation for the top node, node 7 on C1
    [new_bx, new_by] = VertebraMove(theta(7),cen_x,cen_y,dis(7));
    node(iTimeStep,7,1) = new_bx;
    node(iTimeStep,7,2) = new_by;
    [new_vx, new_vy] = VesselMove(new_bx,new_by,theta(7),d);
    vNode(iTimeStep,7,1) = new_vx;
    vNode(iTimeStep,7,2) = new_vy;
    
    % Curve fitting
    ver(2:7) = vNode(iTimeStep,2:7,2);
    hor(2:7) = vNode(iTimeStep,2:7,1);
    cf_= fit(ver',hor',ft_ );
    % Get coefficients from cf_
    coeffs = coeffvalues(cf_);

    % Save coefficient
    for index=1:length(coeffs)
        wmCoeff(iTimeStep,index)=coeffs(index);
    end
end

nodeY = node(:,:,2);
nodeX = node(:,:,1);

vNodeY = vNode(:,:,2);
vNodeX = vNode(:,:,1);

% Save data
save vessel_motion.mat wmCoeff -ascii;
save vessel_node_y.mat vNodeY -ascii;
save vessel_node_x.mat vNodeX -ascii;
save vertebra_node_y.mat nodeY -ascii;
save vertebra_node_x.mat nodeX -ascii;

save vessel_motion.txt wmCoeff -ascii;
save vessel_node_y.txt vNodeY -ascii;
save vessel_node_x.txt vNodeX -ascii;
save vertebra_node_y.txt nodeY -ascii;
save vertebra_node_x.txt nodeX -ascii;
disp('Calculation is finished.')

function d = Distance(p1,p2)
%d = Distance(p1,p2)
% Input;
%      p1: a 1*2 or 2*1 position vector
%      p2: a 1*2 or 2*1 position vector
% Output:
%      d: distance
% This function calculates the distance of the given two vectors.
%
% This function is developed in September 2007
%  Feng Liu
%  Vehicle Safety Division
%  Chalmers University of Technology
%  liu.peak@gmail.com
% Revisions
%--------------------
% February 2009, add comments.
d = ((p1(1)-p2(1))^2+(p1(2)-p2(2))^2)^0.5;

function [x, y] = VertebraMove(theta,cen_x,cen_y,r)
%[x, y] = VertebraRotate(theta,cen_x,cen_y,r)
% This function calculates the new position of the vertebral node.
% Input:
%       theta: rotate angle
%       cen_x: x coordinate of the rotate centre
%       cen_y: y coordinate of the rotate centre
%           r: distance between the rotating vertebra and its rotate centre
% Output:
%        x: x coordinate of the new position
%        y: y coordinate of the new position
%
% This function is developed in September 2007
%  Feng Liu
%  Vehicle Safety Division
%  Chalmers University of Technology
%  liu.peak@gmail.com
% Revisions
%--------------------
% February 2009, rewrite
y = cen_y + r*cos(theta);
x = cen_x - r*sin(theta);

function [x, y] = VesselMove(bx,by,theta,d)
%[x, y] = VesselMove(bx,by,theta,d)
% This funcation returns the new position of the blood vessel.
% Input:
%       bx: x coordinate of the vertebra node
%       by: y coordinate of the vertebra node
%       theta: rotate angle
%       d: distance between cervical vertebra node and the blood vessel
% Output:
%       x: x coordinate of the blood vessel
%       y: y coordinate of the blood vessel
%
% This function is developed in September 2007
%  Feng Liu
%  Vehicle Safety Division
%  Chalmers University of Technology
%  liu.peak@gmail.com
% Revisions
%--------------------
% February 2009, rewrite
x = bx - d * cos(theta);
y = by - d * sin(theta);
